package test;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousServerSocketChannel;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

public abstract class AbstractServer {
    
    protected static class ClientSession {
        private static final AtomicInteger clientCnt = new AtomicInteger(1);
        
        private final int myId = clientCnt.getAndIncrement();
        private volatile int byteCnt;
        
    
        public boolean handleInput(ByteBuffer buf, int len) {
            if (len >= 0) {
                byteCnt += len;
                return true;
            }
            else {               
                System.out.println("received " + byteCnt + " bytes from client " + myId + " by " + Thread.currentThread().getName());
                System.out.flush();   
//                endSignal.countDown();
                return false;
            }
        }
        
        public void handleFailure() {
//            endSignal.countDown();
            System.out.println("*** failure ***  received " + byteCnt + " bytes from client " + myId + " by " + Thread.currentThread().getName());
        }
        
        public String toString() {
            return ("ClientSession " + myId);
        }
    }
    
    
   
//    private static final CountDownLatch endSignal = new CountDownLatch(1024);
    
    protected final AsynchronousServerSocketChannel serverSocketChannel;
    
    public AbstractServer(int serverPort) throws IOException {
        serverSocketChannel = AsynchronousServerSocketChannel.open();
        serverSocketChannel.bind(new InetSocketAddress(serverPort));
    }
    
    public abstract void doAccepting();  
    
}
