package test;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousSocketChannel;
import java.nio.channels.CompletionHandler;

public class Server2 extends AbstractServer {
    private class MyClientSession extends ClientSession {
        private final AsynchronousSocketChannel myChannel;
        
        public MyClientSession(AsynchronousSocketChannel channel) {
            myChannel = channel;
        }
        
        public AsynchronousSocketChannel getChannel() {
            return myChannel;
        }
    }
    
    public Server2(int serverPort) throws IOException {
        super(serverPort);
     }
    
    public void doAccepting() {
        serverSocketChannel.accept(null, new CompletionHandler<AsynchronousSocketChannel,Object>() {

            @Override
            public void completed(AsynchronousSocketChannel channel, Object attachment) {
                
                MyClientSession session = new MyClientSession(channel);

                final ByteBuffer buf = ByteBuffer.allocateDirect(256);
                            
                channel.read(buf, session, new CompletionHandler<Integer, MyClientSession>() {
           
                    @Override
                    public void completed(Integer len, MyClientSession clSession) {

                        if (clSession.handleInput(buf, len)) {
                            clSession.getChannel().read(buf, clSession, this);
                        }
                        else {
                            try {
                                clSession.getChannel().close();
                            } catch (IOException e) { /* ignore */ }
                        }
                    }

                    @Override
                    public void failed(Throwable exc, MyClientSession clSession) {
                        System.out.println("client connection socket problem: " + exc);
                        exc.printStackTrace(); 

                        clSession.handleFailure();
                        
                        try {
                            clSession.getChannel().close();
                        } catch (IOException e) { /* ignore */ }
                    }
                });
                
                serverSocketChannel.accept(null, this);
            }

            @Override
            public void failed(Throwable exc, Object attachment) {
                System.out.println("server socket problem: " + exc);
                exc.printStackTrace();
                
                serverSocketChannel.accept(null, this);
            }
            
        });
    }
   
}
